'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _curry = require('ramda/src/curry');

var _curry2 = _interopRequireDefault(_curry);

var _isEmpty = require('ramda/src/isEmpty');

var _isEmpty2 = _interopRequireDefault(_isEmpty);

var _map = require('ramda/src/map');

var _map2 = _interopRequireDefault(_map);

var _keys = require('ramda/src/keys');

var _keys2 = _interopRequireDefault(_keys);

var _zipObj = require('ramda/src/zipObj');

var _zipObj2 = _interopRequireDefault(_zipObj);

var _prop = require('ramda/src/prop');

var _prop2 = _interopRequireDefault(_prop);

var _reduce = require('ramda/src/reduce');

var _reduce2 = _interopRequireDefault(_reduce);

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _hoistNonReactStatics = require('hoist-non-react-statics');

var _hoistNonReactStatics2 = _interopRequireDefault(_hoistNonReactStatics);

var _validate = require('./validate');

var _validate2 = _interopRequireDefault(_validate);

var _isValid = require('./utils/isValid');

var _isValid2 = _interopRequireDefault(_isValid);

var _debounce = require('./helpers/debounce');

var _debounce2 = _interopRequireDefault(_debounce);

var _updateFormValues = require('./updaters/updateFormValues');

var _updateFormValues2 = _interopRequireDefault(_updateFormValues);

var _updateSyncErrors = require('./updaters/updateSyncErrors');

var _updateSyncErrors2 = _interopRequireDefault(_updateSyncErrors);

var _updateAsyncErrors = require('./updaters/updateAsyncErrors');

var _updateAsyncErrors2 = _interopRequireDefault(_updateAsyncErrors);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }
/* eslint-disable no-nested-ternary */


/**
 * Calculates the new state to be set depending on the actions triggered via the form.
 *
 * Can be an entry point for externally changing the state in future versions, by defining own update functions that
 * change the next state depending on certain user actions. For now this is only an internal implementation.
 *
 * @param {Array} updateFns all update functions that need to change to the current state
 * @param {Object} state the actual state
 * @param {String} type defines which action should be handled via the update functions
 * @param {Object} enhancedProps containing the props as well as additional information like field name or value if available
 * @returns {Object} return the new state that needs to be set.
 */
var runUpdates = function runUpdates(updateFns, state, type, enhancedProps) {
  return (0, _reduce2.default)(function (updatedState, updateFn) {
    return updateFn(updatedState, type, enhancedProps);
  }, [state, []], updateFns

  /**
   * revalidation expects a React Component and returns a React Component containing additional functions and props
   * for managing local component state as well validating that state, wrapping the originally provided Component.
   */
  );
};function revalidation(Component // eslint-disable-line comma-dangle
) {
  var HigherOrderFormComponent = function (_React$Component) {
    _inherits(HigherOrderFormComponent, _React$Component);

    function HigherOrderFormComponent(props) {
      _classCallCheck(this, HigherOrderFormComponent);

      var _this = _possibleConstructorReturn(this, (HigherOrderFormComponent.__proto__ || Object.getPrototypeOf(HigherOrderFormComponent)).call(this, props));

      _initialiseProps.call(_this);

      _this.state = {
        form: (0, _prop2.default)('initialState', props),
        errors: {},
        pending: false,
        debounceFns: HigherOrderFormComponent.createDebounceFunctions((0, _prop2.default)('initialState', props))
      };
      return _this;
    }

    _createClass(HigherOrderFormComponent, [{
      key: 'componentWillReceiveProps',
      value: function componentWillReceiveProps(_ref) {
        var updateForm = _ref.updateForm;

        if (updateForm) {
          this.updateState(updateForm);
        }
      }
    }, {
      key: 'render',
      value: function render() {
        var _state = this.state,
            form = _state.form,
            errors = _state.errors,
            pending = _state.pending,
            debounceFns = _state.debounceFns;
        /* eslint-disable no-unused-vars */

        var _props = this.props,
            rules = _props.rules,
            asyncRules = _props.asyncRules,
            initialState = _props.initialState,
            updateForm = _props.updateForm,
            validateSingle = _props.validateSingle,
            instantValidation = _props.instantValidation,
            rest = _objectWithoutProperties(_props, ['rules', 'asyncRules', 'initialState', 'updateForm', 'validateSingle', 'instantValidation']);

        var valid = (0, _isValid2.default)((0, _validate2.default)(rules, form)) && (0, _isValid2.default)(errors);

        var reValidation = {
          form: form,
          errors: errors,
          pending: pending,
          valid: valid,
          debounce: debounceFns,
          updateState: this.updateState,
          updateValue: this.updateValue,
          validateAll: this.validateAll
        };

        return (0, _react.createElement)(Component, _extends({}, rest, {
          reValidation: reValidation
        }));
      }
    }], [{
      key: 'createDebounceFunctions',
      value: function createDebounceFunctions(form) {
        return (0, _zipObj2.default)((0, _keys2.default)(form), (0, _map2.default)(_debounce2.default, (0, _keys2.default)(form)));
      }
    }]);

    return HigherOrderFormComponent;
  }(_react2.default.Component);

  HigherOrderFormComponent.defaultProps = {
    initialState: {},
    validateSingle: true,
    instantValidation: true
  };

  var _initialiseProps = function _initialiseProps() {
    var _this2 = this;

    this.updateFns = [_updateFormValues2.default, _updateSyncErrors2.default, _updateAsyncErrors2.default];

    this.validateAll = function (cb, data) {
      var effects = [];
      var updatedState = [];
      _this2.setState(function (state, props) {
        var _runUpdates = runUpdates(_this2.updateFns, state, [_constants.VALIDATE_ALL], _extends({}, props));

        var _runUpdates2 = _slicedToArray(_runUpdates, 2);

        updatedState = _runUpdates2[0];
        effects = _runUpdates2[1];

        return updatedState;
      }, function () {
        if ((0, _isEmpty2.default)(effects)) {
          if ((0, _isValid2.default)(_this2.state.errors) && cb) cb(data || _this2.state.form);
        } else {
          (0, _map2.default)(function (f) {
            return f().fork(function () {}, function (x) {
              return _this2.setState(x, function () {
                if ((0, _isValid2.default)(_this2.state.errors) && cb) cb(data || _this2.state.form);
              });
            });
          }, effects);
        }
      } // eslint-disable-line comma-dangle
      );
    };

    this.updateState = function (newState) {
      var effects = [];
      var updatedState = [];
      var getType = function getType(_ref2) {
        var instantValidation = _ref2.instantValidation;
        return instantValidation ? [_constants.UPDATE_ALL, _constants.VALIDATE_ALL] : [_constants.UPDATE_ALL];
      };

      _this2.setState(function (state, props) {
        var _runUpdates3 = runUpdates(_this2.updateFns, state, getType(props), _extends({}, props, { value: newState }));

        var _runUpdates4 = _slicedToArray(_runUpdates3, 2);

        updatedState = _runUpdates4[0];
        effects = _runUpdates4[1];

        return updatedState;
      }, function () {
        return (0, _map2.default)(function (f) {
          return f().fork(function () {}, function (x) {
            return _this2.setState(x);
          });
        }, effects // eslint-disable-line comma-dangle
        );
      });
    };

    this.updateValue = (0, _curry2.default)(function (name, value) {
      var type = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

      var effects = [];
      var updatedState = [];
      var getType = function getType(_ref3) {
        var instantValidation = _ref3.instantValidation,
            validateSingle = _ref3.validateSingle;
        return instantValidation ? validateSingle ? [_constants.UPDATE_FIELD, _constants.VALIDATE_FIELD] : [_constants.UPDATE_FIELD, _constants.VALIDATE_ALL] : [_constants.UPDATE_FIELD];
      };

      _this2.setState(function (state, props) {
        var _runUpdates5 = runUpdates(_this2.updateFns, state, type ? [type] : getType(props), _extends({}, props, { name: name, value: value }));

        var _runUpdates6 = _slicedToArray(_runUpdates5, 2);

        updatedState = _runUpdates6[0];
        effects = _runUpdates6[1];

        return updatedState;
      }, function () {
        return (0, _map2.default)(function (f) {
          return f().fork(function () {}, function (result) {
            return _this2.setState(result);
          });
        }, effects // eslint-disable-line comma-dangle
        );
      });
    });
  };

  HigherOrderFormComponent.displayName = 'Revalidation_(' + (Component.displayName || Component.name || 'Component') + ')';
  return (0, _hoistNonReactStatics2.default)(HigherOrderFormComponent, Component);
}

exports.default = revalidation;